function B = arnold2k(A, k, M)
%%ARNOLD2 Applica k passi della mappa di Arnold. 
% Questa funziona applica K passi della mappa del gatto di Arnold 
% all'immagine rappresentata dalla matrice A. Se viene specificato
% anche il parametro opzionale M (una matrice 2x2), questo viene
% utilizzato al posto di [2 1 ; 1 1] per definire la mappa.

m = size(A, 1);

if ~exist('M', 'var')
    M = [ 2 1 ; 1 1 ];
end

N = eye(2);

for j = 1 : k
    N = mod(M * N, m);
end

% Costruiamo una matrice 2 x m^2 che contiene tutti gli indici (i,j)
II = zeros(2, m^2);
II(2, :) = kron(1:m, ones(1,m));
II(1, :) = kron(ones(1,m), 1:m);

IIh = 1 + mod(N * (II - 1), m);

% Costruiamo un vettore 1 x m^2 che corrisponde agli indici in IIh
Ih = [ 1 , m ] * (IIh - [ 0 ; 1 ] * ones(1, m^2));

B = A;
for j = 1 : size(B, 3)
    B(Ih + (j-1)*m^2) = reshape(A(:,:,j), m*m, 1);
end

end



